﻿-- KBM: Mark-It 
-- Written by Paul Snart
-- Copyright 2012

KBMMI_Settings = nil
chKBMMMI_Settings = nil

local AddonIni = ...

local AddonData = Inspect.Addon.Detail("KingMolinator")

local LSUIni = Inspect.Addon.Detail("SafesUnitLib")
local LibSUnit = LSUIni.data

local LSCIni = Inspect.Addon.Detail("SafesCastbarLib")
local LibSCast = LSCIni.data

local PI = {
	Context = UI.CreateContext("KBM_MarkIt"),
	ID = "KBMMarkIt",
	Name = "Mark-It",
	APIData = Inspect.Addon.Detail("KBMMarkIt"),
	Menu = {},
	Lang = {},
	MaxMarks = 30,
	Hide = true,
	MenuOptions = {
		Settings = {},
	},
	Frame = {},
	Stores = {
		Frames = {},
	},
	Constant = {
		Frame = {
			Width = 200,
			Height = 50,
			TextSize = 12,
			Inactive = 0.2,
			Active = 1,
		},
	},
	MinVersion = {
		High = 1,
		Mid = 5,
		Low = 5,
		Revision = 758,
	},
	LastTime = 0,
}
PI.Context:SetMouseMasking("limited")

if not AddonData then	
	print("King Boss Mods not installed, please install the latest version."..
			"\nKBM-MI01: Plug-In link failed.")
	return
end
local KBM = AddonData.data
if not KBM then
	print("Incorrect version of King Boss Mods installed. Please install the latest version."..
			"\nKBM-MI02: Plug-In link failed.")
	return
end

local MICastCount = 0
local MarkIt_CastSettings = LibSCast.Default.BarSettings()
MarkIt_CastSettings.pack = "Simple"
MarkIt_CastSettings.texture.foreground.enabled = false
MarkIt_CastSettings.unlocked = false
MarkIt_CastSettings.visible = false

PI.Context:SetLayer(KBM.Context:GetLayer()+1)

-- Addon Dictionary
-- Frame Related
PI.Lang.Frames = {}
PI.Lang.Frames.Interact = KBM.Language:Add("Interactive Mode.")
PI.Lang.Frames.Interact:SetGerman("Interaktiver Modus.")
PI.Lang.Frames.Interact:SetFrench("Mode Interactif.")
PI.Lang.Frames.Floating = KBM.Language:Add("Unpin from NumAnchor.")
PI.Lang.Frames.Floating:SetGerman("Abheften vom Haupt-Anker.")
PI.Lang.Frames.Floating:SetFrench("Détacher de l'ancrage.")
PI.Lang.Frames.UnlockFrame = KBM.Language:Add("Unlock Frame (for positioning)")
PI.Lang.Frames.UnlockFrame:SetGerman("Fenster ist verschiebbar")
PI.Lang.Frames.UnlockFrame:SetFrench("Débloquer cadre (pour positionnement).")

-- General Settings
PI.Lang.Options = {}
PI.Lang.Options.Fade = KBM.Language:Add("Fade when inactive")
PI.Lang.Options.Fade:SetFrench("Pâlir quand inactif")
PI.Lang.Options.Fade:SetGerman("Fenster verblassen, wenn nicht aktiv")
PI.Lang.Options.Auto = KBM.Language:Add("Auto hide while playing solo.")
PI.Lang.Options.Auto:SetGerman("Automatisch verstecken während dem Solospiel")
PI.Lang.Options.Auto:SetFrench("Cacher automatiquement en jeu solo.")

-- Pin Related
PI.Lang.Pin = {}
PI.Lang.Pin.Visible = KBM.Language:Add("Unlock Pin NumAnchor (for positioning)")
PI.Lang.Pin.Visible:SetGerman("Haupt-Anker ist verschiebbar.")
PI.Lang.Pin.Visible:SetFrench("Débloquer de l'ancrage principale (pour positionnement)")

-- Error Related
PI.Lang.Error = {}
PI.Lang.Error.Install = KBM.Language:Add("Incorrect version of King Boss Mods installed. Please install the latest version.")
PI.Lang.Error.Install:SetGerman("Veraltete Version von King Boss Mods gefunden. Bitte installiert die letzte Version.")
PI.Lang.Error.Install:SetFrench("Version incorrecte de King Boss Mods installée. SVP installer la dernière version.")

PI.MarkName = KBM.Marks.Name

if not KBM.Version:Check(PI.MinVersion.High, PI.MinVersion.Mid, PI.MinVersion.Low, PI.MinVersion.Revision) then
	print(PI.Name.." is expecting at least KBM v"..PI.MinVersion.High.."."..PI.MinVersion.Mid.."."..PI.MinVersion.Low..
			"\n"..PI.Lang.Error.Install[KBM.Lang].."\nKBM-MIV01: Version Incompatible")
	return
end

if KBM.PlugIn.Register then
	KBM.PlugIn.Register(PI)
else
	print(PI.Lang.Error.Install[KBM.Lang].."\nKBM-MI03: Plug-In link failed.")
	return
end

PI.Version = {}
function PI.Version:Load()
	local s = 0
	local e = 0
	s, e, self.High, self.Mid, self.Low, self.Revision = string.find(AddonIni.toc.Version, "(%d+).(%d+).(%d+).(%d+)")
	if self.High then
		self.High = tonumber(self.High)
	else
		self.High = 0
	end
	if self.Mid then
		self.Mid = tonumber(self.Mid)
	else
		self.Mid = 0
	end
	if self.Low then
		self.Low = tonumber(self.Low)
	else
		self.Low = 0
	end
	if self.Revision then
		self.Revision = tonumber(self.Revision)
	else
		self.Revision = 0
	end
end
PI.Version:Load()

function PI.Version:Check(Rev)
	if self.Revision < Rev then
		return false
	else
		return true
	end
end

PI.Defaults = {}
function PI.Defaults.Frame()
	local FrameObj = {
		Type = "Frame",
		Enabled = true,
		Floating = false,
		PinObject = 1,
		Style = "kbm",
		Interact = true,
		Visible = true,
		ScaleText = false,
		ScaleWidth = false,
		ScaleHeight = false,
		wScale = 1,
		hScale = 1,
		tScale = 1,
		x = false,
		y = false,
	}
	return FrameObj
end

function PI.Defaults.Unit()
	local UnitObj = {
		Type = "Unit",
	}
	return UnitObj
end

function PI:InitVars()
	self.Context:SetSecureMode("restricted")
	self.Settings = {
		Enabled = true,
		Frame = {},
		AutoHide = true,
		NumAnchor = self.Defaults.Frame(),
		Fade = true,
	}
	for i = 1, PI.MaxMarks do
		self.Settings.Frame[i] = self.Defaults.Frame()
	end
	
	KBMMI_Settings = self.Settings
	chKBMMI_Settings = self.Settings
	
	self.Pins = {}
	
end

function PI:SwapSettings(bool)
	if bool then
		KBMMI_Settings = self.Settings
		self.Settings = chKBMMI_Settings
	else
		chKBMMI_Settings = self.Settings
		self.Settings = KBMMI_Settings
	end
end

function PI:LoadVars()
	if KBM.Options.Character then
		KBM.LoadTable(chKBMMI_Settings, self.Settings)
	else
		KBM.LoadTable(KBMMI_Settings, self.Settings)
	end
		
	if KBM.Options.Character then
		chKBMMI_Settings = self.Settings
	else
		KBMMI_Settings = self.Settings
	end
	self.Settings.NumAnchor.Type = "NumAnchor"
	self.Hide = self.Settings.AutoHide
	
	self:InitFrames()
end

function PI:SaveVars()
	if KBM.Options.Character then
		chKBMMI_Settings = self.Settings
	else
		KBMMI_Settings = self.Settings
	end
end

function PI:SetPins()
	if not self.Secure then
		self.Pins.Last = nil
		self.Pins.First = nil
		self.Pins.Total = 0
		for Index, Frame in ipairs(self.Frame) do
			Frame.GUI.Input:ClearAll()
			Frame.GUI:ApplySettings()
			if Frame.Settings.Enabled and PI.Settings.Enabled and PI.Hide == false then
				if not Frame.Settings.Floating then
					if not self.Pins.Last then
						self.Pins.Last = Frame
						self.Pins.First = Frame
						Frame.GUI.Input:SetPoint("TOPLEFT", self.GUI.NumAnchor.Input, "TOPLEFT")
					else
						Frame.GUI.Input:SetPoint("TOPLEFT", self.Pins.Last.GUI.Unit.Frame, "BOTTOMLEFT", 0, 0)
						self.Pins.Last = Frame
					end
					self.Pins.Total = self.Pins.Total + 1
				else
					if Frame.Settings.x then
						Frame.GUI.Input:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Frame.Settings.x, Frame.Settings.y)
					else
						Frame.GUI.Input:SetPoint("CENTER", UIParent, "CENTER")
					end
				end
				Frame.GUI.Interact:SetVisible(true)
				Frame.GUI.IntDisplay:SetVisible(true)
				if Frame.Unit then
					Frame.GUI.IntDisplay:SetAlpha(PI.Constant.Frame.Active)
				else
					if PI.Settings.Fade then
						Frame.GUI.IntDisplay:SetAlpha(PI.Constant.Frame.Inactive)
					else
						Frame.GUI.IntDisplay:SetAlpha(PI.Constant.Frame.Active)
					end
				end
				if KBM.Menu.Active then
					if Frame.Settings.Floating then
						Frame.GUI.Input:SetVisible(Frame.Settings.Visible)
					end
				end
			else
				Frame.GUI.Interact:SetVisible(false)
				Frame.GUI.IntDisplay:SetVisible(false)
				Frame.GUI.Input:SetVisible(false)
				Frame.GUI.Unit.Frame:SetVisible(false)
			end
		end
	end
end

function PI:PullFrame()
	local GUI = nil
	if #self.Stores.Frames > 0 then
		GUI = table.remove(self.Stores.Frames)
	else
		GUI = {}
		GUI.Input = UI.CreateFrame("Frame", "KBMMI_Input_Frame", KBM.Context)
		GUI.Input:SetLayer(5)
		GUI.Input:SetBackgroundColor(0,0,0,0.33)
		GUI.Input:SetVisible(false)
		GUI.Input:SetMouseMasking("limited")
		GUI.IntDisplay = UI.CreateFrame("Texture", "KBMMI_Interactive_SelectFrame", KBM.Context)
		GUI.IntDisplay:SetBackgroundColor(0,0,0,0.33)
		GUI.IntDisplay:SetTexture("KingMolinator", "Media/BarSkin.png")
		GUI.IntDisplay:SetLayer(2)
		GUI.IntDisplay:SetMouseMasking("limited")
		GUI.Interact = UI.CreateFrame("Frame", "KBMMI_Interactive_Frame", self.Context)
		GUI.Interact:SetSecureMode("restricted")
		GUI.Interact:SetLayer(5)
		GUI.Interact:SetPoint("TOPLEFT", GUI.Input, "TOPLEFT")
		GUI.Interact:SetPoint("BOTTOMRIGHT", GUI.Input, "BOTTOMRIGHT")
		GUI.Interact:SetMouseMasking("limited")
		GUI.IntDisplay:SetPoint("TOPLEFT", GUI.Input, "TOPLEFT")
		GUI.IntDisplay:SetPoint("BOTTOMRIGHT", GUI.Input, "BOTTOMRIGHT")
		GUI.IntIcon = UI.CreateFrame("Texture", "KBMMI_LargeIcon", GUI.IntDisplay)
		GUI.IntIcon:SetPoint("TOPLEFT", GUI.Input, 0.38, 0.06)
		GUI.IntIcon:SetPoint("BOTTOMRIGHT", GUI.Input, 0.62, 0.94)
		GUI.Unit = {}
		GUI.Unit.Frame = UI.CreateFrame("Frame", "KBMMI_NonInteractive_UnitDisplay", KBM.Context)
		GUI.Unit.Frame:SetLayer(3)
		GUI.Unit.Frame:SetPoint("TOPLEFT", GUI.Input, "TOPLEFT")
		GUI.Unit.Frame:SetPoint("BOTTOMRIGHT", GUI.Input, "BOTTOMRIGHT")
		GUI.Unit.Frame:SetBackgroundColor(0,0,0,0)
		GUI.Unit.Frame:SetMouseMasking("limited")
		GUI.Unit.HP = UI.CreateFrame("Texture", "KBMMI_UnitHPBar", GUI.Unit.Frame)
		GUI.Unit.HP:SetTexture("KingMolinator", "Media/BarTexture.png")
		GUI.Unit.HP:SetBackgroundColor(0,1,0,0.3)
		GUI.Unit.HP:SetWidth(GUI.Unit.Frame:GetWidth())
		GUI.Unit.HP:SetPoint("TOPLEFT", GUI.Unit.Frame, "TOPLEFT")
		GUI.Unit.HP:SetPoint("BOTTOM", GUI.Unit.Frame, "CENTERY")
		GUI.Unit.Icon = UI.CreateFrame("Texture", "KBMMI_MarkIcon", GUI.Unit.Frame)
		GUI.Unit.Icon:SetPoint("TOPLEFT", GUI.Unit.HP, 0.01, 0.1)
		GUI.Unit.Icon:SetPoint("BOTTOM", GUI.Unit.HP, nil, 0.9)
		GUI.Unit.Icon:SetWidth(GUI.Unit.Icon:GetHeight())
		GUI.Unit.Icon:SetLayer(2)
		GUI.Unit.NameShadow = UI.CreateFrame("Text", "KBMMI_NameShadow", GUI.Unit.Frame)
		GUI.Unit.NameShadow:SetFontColor(0,0,0,1)
		GUI.Unit.NameShadow:SetPoint("CENTERLEFT", GUI.Unit.Icon, "CENTERRIGHT", 0, 1)
		GUI.Unit.NameShadow:SetLayer(2)
		GUI.Unit.Name = UI.CreateFrame("Text", "KBMMI_Name", GUI.Unit.NameShadow)
		GUI.Unit.Name:SetFontColor(1,1,1,1)
		GUI.Unit.Name:SetPoint("TOPLEFT", GUI.Unit.NameShadow, "TOPLEFT", -1, -1)
		GUI.Unit.Name:SetLayer(3)
		GUI.Unit.PercentShadow = UI.CreateFrame("Text", "KBMMI_PercentShadow", GUI.Unit.Frame)
		GUI.Unit.PercentShadow:SetFontColor(0,0,0,1)
		GUI.Unit.PercentShadow:SetLayer(2)
		GUI.Unit.PercentShadow:SetPoint("RIGHT", GUI.Unit.Frame, "RIGHT")
		GUI.Unit.PercentShadow:SetPoint("TOP", GUI.Unit.NameShadow, "TOP")
		GUI.Unit.Percent = UI.CreateFrame("Text", "KBMMI_Percent", GUI.Unit.PercentShadow)
		GUI.Unit.Percent:SetPoint("TOPLEFT", GUI.Unit.PercentShadow, "TOPLEFT", -1, -1)
		GUI.Unit.Percent:SetLayer(3)
		GUI.Unit.Percent:SetFontColor(1,1,1,1)
		GUI.Unit.Frame:SetVisible(false)
		if PI.Settings.Fade then
			GUI.IntDisplay:SetAlpha(PI.Constant.Frame.Inactive)
		end
		
		function GUI.Interact.Event:MouseIn()
			if PI.Active then
				if not KBM.Menu.Active then
					GUI.IntDisplay:SetAlpha(PI.Constant.Frame.Active)
				end
			end
		end
		
		function GUI.Interact.Event:MouseOut()
			if not GUI.Unit.Frame:GetVisible() then
				if PI.Settings.Fade then
					GUI.IntDisplay:SetAlpha(PI.Constant.Frame.Inactive)
				end
			end
		end
		
		function GUI:SetIcon(Icon, SmallIcon, Location)
			Location = Location or "Rift"
			self.IntIcon:SetTexture("Rift", Icon)
			self.Unit.Icon:SetTexture(Location, SmallIcon) 
		end
				
		function GUI.Unit:SetName(Text)
			self.NameShadow:SetText(Text)
			self.Name:SetText(Text)
		end
		
		function GUI.Unit:SetPercent(Text)
			self.PercentShadow:SetText(Text.."%")
			self.Percent:SetText(Text.."%")
		end
		
		function GUI:Move(Type)
			if Type == "end" then
				self.Settings.x = self.Input:GetLeft()
				self.Settings.y = self.Input:GetTop()
			end
		end
		
		GUI.Drag = KBM.AttachDragFrame(GUI.Input, function (Type) GUI:Move(Type) end, "KBMMI_Drag")

		function GUI:Update()
			if self.Unit then
				local Width = self.Interact:GetWidth()
				if self.Frame.Unit then
					self.Unit.HP:SetWidth(self.Frame.Unit.KBMUnit.PercentRaw * Width)
					self.Unit:SetPercent(self.Frame.Unit.KBMUnit.PercentFlat)
				end
			end
		end
		
		function GUI:ApplySettings()
			if self.Settings.Floating then
				self.Input:SetWidth(PI.Constant.Frame.Width * self.Settings.wScale)
				self.Input:SetHeight(PI.Constant.Frame.Height * self.Settings.hScale)
				self.Unit.Name:SetFontSize(PI.Constant.Frame.TextSize * self.Settings.tScale)
			else
				self.Input:SetWidth(PI.Constant.Frame.Width * PI.Settings.NumAnchor.wScale)
				self.Input:SetHeight(PI.Constant.Frame.Height * PI.Settings.NumAnchor.hScale)
				self.Unit.Name:SetFontSize(PI.Constant.Frame.TextSize * PI.Settings.NumAnchor.tScale)
			end
			self.Unit.NameShadow:SetFontSize(self.Unit.Name:GetFontSize())
			self.Unit.Percent:SetFontSize(self.Unit.Name:GetFontSize())
			self.Unit.PercentShadow:SetFontSize(self.Unit.Name:GetFontSize())
			self.Unit.Icon:SetWidth(self.Unit.Icon:GetHeight())
			GUI:Update()
		end
		
		function GUI.Drag.Event:WheelForward()	
			local Change = false
			if self.Settings.ScaleWidth then
				if self.Settings.wScale < 1.6 then
					self.Settings.wScale = self.Settings.wScale + 0.02
					if self.Settings.wScale > 1.6 then
						self.Settings.wScale = 1.6
					end
					Change = true
				end
			end
			
			if self.Settings.ScaleHeight then
				if self.Settings.hScale < 1.6 then
					self.Settings.hScale = self.Settings.hScale + 0.02
					if self.Settings.hScale > 1.6 then
						self.Settings.hScale = 1.6
					end
					Change = true
				end
			end
			
			if self.Settings.ScaleText then
				if self.Settings.tScale < 1.6 then
					self.Settings.tScale = self.Settings.tScale + 0.02
					if self.Settings.tScale > 1.6 then
						self.Settings.tScale = 1.6
					end
					Change = true
				end
			end			
			
			if self.Settings.Type == "NumAnchor" then
				GUI:ApplySettings()
				PI:SetPins()
			else
				GUI:ApplySettings()
			end
		end
		
		function GUI.Drag.Event:WheelBack()	
			local Change = false
			if self.Settings.ScaleWidth then
				if self.Settings.wScale > 0.6 then
					self.Settings.wScale = self.Settings.wScale - 0.02
					if self.Settings.wScale < 0.6 then
						self.Settings.wScale = 0.6
					end
					Change = true
				end
			end
			
			if self.Settings.ScaleHeight then
				if self.Settings.hScale > 0.6 then
					self.Settings.hScale = self.Settings.hScale - 0.02
					if self.Settings.hScale < 0.6 then
						self.Settings.hScale = 0.6
					end
					Change = true
				end
			end
			
			if self.Settings.ScaleText then
				if self.Settings.tScale > 0.6 then
					self.Settings.tScale = self.Settings.tScale - 0.02
					if self.Settings.tScale < 0.6 then
						self.Settings.tScale = 0.6
					end
					Change = true
				end
			end
			
			if Change then
				if self.Settings.Type == "NumAnchor" then
					GUI:ApplySettings()
					PI:SetPins()
				else
					GUI:ApplySettings()
				end
			end
		end
		
		MICastCount = MICastCount + 1
		GUI.Castbar = LibSCast:Create("AddWatch_Castbar_"..MICastCount, GUI.Unit.Frame, "Simple", MarkIt_CastSettings, "default")
		
	end
	
	return GUI
end

function PI:InitFrames()
	self.GUI = {}
	self.GUI.NumAnchor = self:PullFrame()
	self.GUI.NumAnchor.Input:SetWidth(PI.Constant.Frame.Width * self.Settings.NumAnchor.wScale)
	self.GUI.NumAnchor.Input:SetHeight(PI.Constant.Frame.Height * self.Settings.NumAnchor.hScale)
	
	if not self.Settings.NumAnchor.x then
		self.GUI.NumAnchor.Input:SetPoint("TOPLEFT", UIParent, 0.75, 0.25)
	else
		self.GUI.NumAnchor.Input:SetPoint("TOPLEFT", UIParent, "TOPLEFT", self.Settings.NumAnchor.x, self.Settings.NumAnchor.y)
	end
	
	self.GUI.NumAnchor.Input:SetVisible(false)
	self.GUI.NumAnchor.Frame = self
	self.GUI.NumAnchor.Drag.Frame = self
	self.GUI.NumAnchor.Settings = self.Settings.NumAnchor
	self.GUI.NumAnchor.Drag.Settings = self.Settings.NumAnchor
	self.GUI.NumAnchor.Interact:SetVisible(false)
	self.GUI.NumAnchor.IntDisplay:SetVisible(false)

	for Mark = 1, PI.MaxMarks do
		self.Frame[Mark] = {
			Mod = PI,
			Mark = Mark,
			Name = "KBMMarkIt_Frame"..Mark,
			Settings = PI.Settings.Frame[Mark],
			MenuOptions = {},
			Macros = {
				Left = "/target @mark "..Mark,
				Right = "/mark "..Mark,
			},
			GUI = self:PullFrame(),
		}
		local Frame = self.Frame[Mark]
		Frame.GUI.Frame = Frame
		Frame.GUI.Drag.Frame = Frame
		Frame.GUI.Settings = Frame.Settings
		Frame.GUI.Drag.Settings = Frame.Settings
		Frame.GUI.Interact.Event.LeftClick = Frame.Macros.Left
		Frame.GUI.Interact.Event.RightClick = Frame.Macros.Right
		Frame.GUI:SetIcon(KBM.Marks.FileFull[Mark], KBM.Marks.File[Mark], KBM.Marks.Location[Mark])
		function Frame:Set(Unit)
			if self.Settings.Enabled and PI.Settings.Enabled and PI.Hide == false then
				self.GUI.IntIcon:SetVisible(false)
				self.GUI.Unit.Frame:SetVisible(true)
				self.GUI.IntDisplay:SetAlpha(PI.Constant.Frame.Active)
				self.GUI.Castbar:Start(Unit.UnitID)
			else
				self.GUI.Unit.Frame:SetVisible(false)
			end
			self.Unit = Unit
			local Width = self.GUI.Interact:GetWidth()
			--self.GUI.Unit.HP:SetWidth((self.Unit.KBMUnit.PercentRaw * 0.01) * Width)
			self.GUI.Unit:SetName(self.Unit.KBMUnit.Name)
			self.GUI:Update()
		end
		function Frame:Clear()
			if self.Settings.Enabled then
				if self.GUI then
					self.GUI.Castbar:Stop()
					self.GUI.IntIcon:SetVisible(true)
					self.GUI.Unit.Frame:SetVisible(false)
					if PI.Settings.Fade then
						self.GUI.IntDisplay:SetAlpha(PI.Constant.Frame.Inactive)
					end
				end
			else
				self.GUI.Castbar:Stop()
				if self.GUI then
					self.GUI.IntIcon:SetVisible(false)
					self.GUI.Unit.Frame:SetVisible(false)
					self.GUI.IntDisplay:SetVisible(false)
				end
			end
			self.Unit = nil
		end
	end
	
	self.UnitTrack = {}
	self:SetPins()
end

function PI:InitCastBars()
	for Mark, Frame in ipairs(self.Frame) do
		function Frame.PinCastbar(self)
			self.ui.cradle:ClearAll()
			self.ui.cradle:SetPoint("TOPLEFT", Frame.GUI.Unit.HP, "BOTTOMLEFT")
			self.ui.cradle:SetPoint("RIGHT", Frame.GUI.IntDisplay, "RIGHT")
			self.ui.cradle:SetPoint("BOTTOM", Frame.GUI.IntDisplay, "BOTTOM")
			self.ui.text:SetFontSize(PI.Constant.Frame.TextSize * self.Settings.scale.t)
			self.ui.cradle:SetLayer(6)
		end
		Frame.GUI.Castbar:Pin(Frame.PinCastbar)
	end
end

function PI.MarkChange(handle, uList)
	self = PI
	for UnitID, UnitObj in pairs(uList) do
		local Mark = UnitObj.Mark
		if self.Settings.Enabled then
			if Mark then
				Mark = tonumber(Mark)
				if self.UnitTrack[UnitID] then
					if self.UnitTrack[UnitID].Mark ~= Mark then
						if self.UnitTrack[UnitID].Mark then
							if self.Frame[self.UnitTrack[UnitID].Mark].Unit == self.UnitTrack[UnitID] then
								self.Frame[self.UnitTrack[UnitID].Mark]:Clear()
							end
						end
						self.UnitTrack[UnitID].Mark = Mark
						self.UnitTrack[UnitID].Frame = self.Frame[Mark]
						self.UnitTrack[UnitID].KBMUnit = UnitObj
					end
				else
					self.UnitTrack[UnitID] = self.Defaults.Unit()
					self.UnitTrack[UnitID].UnitID = UnitID
					self.UnitTrack[UnitID].Mark = Mark
					self.UnitTrack[UnitID].Frame = self.Frame[Mark]
					self.UnitTrack[UnitID].KBMUnit = UnitObj
				end
				self.Frame[Mark]:Set(self.UnitTrack[UnitID])
			else
				if UnitID then
					if self.UnitTrack[UnitID] then
						if self.Frame[self.UnitTrack[UnitID].Mark].Unit == self.UnitTrack[UnitID] then
							self.Frame[self.UnitTrack[UnitID].Mark]:Clear()
							self.UnitTrack[UnitID] = nil
						end
					end
				end
			end
		end
	end
end

function PI.UpdateUnit(handle, UnitObj)
	if PI.UnitTrack[UnitObj.UnitID] then
		PI.UnitTrack[UnitObj.UnitID].Frame.GUI:Update()
	end
end

function PI.UnitAvailable(handle, UnitObj)
	if UnitObj.Mark then
		PI.MarkChange(handle, {[UnitObj.UnitID] = UnitObj})
	end
end

function PI.UnitUnavailable(handle, UnitObj)
end

function PI.UnitName(handle, uList)
	for UnitID, UnitObj in pairs(uList) do
		if PI.UnitTrack[UnitID] then
			PI.MarkChange(handle, {[UnitID] = UnitObj})
		end
	end
end

function PI:Timer(current)
end

function PI:InitFrameMenus()
	for Index, Frame in ipairs(self.Frame) do
		
		local function BuildOptions()
			local Callbacks = {}
			function Callbacks:Enabled(bool)
				Frame.Settings.Enabled = bool
				PI:SetPins()
			end
			function Callbacks:Interact(bool)
				Frame.Settings.Interact = bool
			end
			function Callbacks:Floating(bool)
				Frame.Settings.Floating = bool
				PI:SetPins()
			end
			function Callbacks:Visible(bool)
				Frame.Settings.Visible = bool
				Frame.GUI.Input:SetVisible(bool)
			end
			function Callbacks:ScaleWidth(bool)
				Frame.Settings.ScaleWidth = bool
			end
			function Callbacks:ScaleHeight(bool)
				Frame.Settings.ScaleHeight = bool
			end
			function Callbacks:ScaleText(bool)
				Frame.Settings.ScaleText = bool
			end
			
			local Options = Frame.Menu.Settings
			
			Frame.Menu.Enabled = Options.UI.CreateHeader(KBM.Language.Options.Enabled[KBM.Lang], Frame.Settings, "Enabled", Callbacks)
			--Menu= self.Enabled:AddCheck(PI.Lang.Frames.Interact[KBM.Lang], self.Callbacks.Interact, self.Frame.Settings.Interact)
			--Menu:Disable()
			Frame.Menu.Floating = Frame.Menu.Enabled:CreateHeader(PI.Lang.Frames.Floating[KBM.Lang], Frame.Settings, "Floating", Callbacks)
			Frame.Menu.Floating:CreateCheck(PI.Lang.Frames.UnlockFrame[KBM.Lang], Frame.Settings, "Visible", Callbacks)
			Frame.Menu.Floating:CreateCheck(KBM.Language.Options.UnlockWidth[KBM.Lang], Frame.Settings, "ScaleWidth", Callbacks)
			Frame.Menu.Floating:CreateCheck(KBM.Language.Options.UnlockHeight[KBM.Lang], Frame.Settings, "ScaleHeight", Callbacks)
			Frame.Menu.Floating:CreateCheck(KBM.Language.Options.UnlockText[KBM.Lang], Frame.Settings, "ScaleText", Callbacks)
		end
		
		-- function Callbacks:exEnabled(bool)
			-- if not Frame.Sleep then
				-- Frame.Sleep = true
				-- if Frame.MenuOptions.Enabled then
					-- if Frame.MenuOptions.Enabled.GUI then
						-- if Frame.MenuOptions.Enabled.GUI.Check then
							-- Frame.MenuOptions.Enabled.GUI.Check:SetChecked(bool)
						-- end
					-- end
				-- end
				-- Frame.Settings.Enabled = bool
				-- PI:SetPins()
				-- Frame.Sleep = false
			-- end
		-- end
		
		Frame.Menu = {}
		Frame.Menu.Settings = self.Menu.Header:CreateItem("Mark Frame "..PI.MarkName[Index], "MARK_FRAME_"..string.upper(PI.MarkName[Index]))
		BuildOptions()
		-- Frame.Menu:SetHook(Frame.Callbacks.exEnabled)		
	end
end

function PI:MenuOpen()
	for Index, Frame in ipairs(self.Frame) do
		if self.Settings.Enabled and Frame.Settings.Enabled then
			if Frame.Settings.Floating then
				Frame.GUI.Input:SetVisible(Frame.Settings.Visible)
			end
		else
			Frame.GUI.Input:SetVisible(false)
		end
	end
	if self.Settings.Enabled then
		self.GUI.NumAnchor.Input:SetVisible(self.Settings.NumAnchor.Visible)
	end
end

function PI:MenuClose()
	for Index, Frame in ipairs(self.Frame) do
		Frame.GUI.Input:SetVisible(false)
	end
	self.GUI.NumAnchor.Input:SetVisible(false)
end

function PI.Menu:BuildOptions()
	self.Callbacks = {}
	function self.Callbacks:Enabled(bool)
		PI.Settings.Enabled = bool
		PI.GUI.NumAnchor.Input:SetVisible(bool)
		PI:SetPins()
	end
	function self.Callbacks:Fade(bool)
		PI.Settings.Fade = bool
		PI:SetPins()
	end
	function self.Callbacks:AutoHide(bool)
		PI.Settings.AutoHide = bool
		if not PI.Secure then
			if not LibSUnit.Raid.Grouped then
				PI.Hide = bool
				PI:SetPins()
			end
		end
	end
	function self.Callbacks:Visible(bool)
		PI.Settings.NumAnchor.Visible = bool
		PI.GUI.NumAnchor.Input:SetVisible(bool)
	end
	function self.Callbacks:ScaleWidth(bool)
		PI.Settings.NumAnchor.ScaleWidth = bool
	end
	function self.Callbacks:ScaleHeight(bool)
		PI.Settings.NumAnchor.ScaleHeight = bool
	end
	function self.Callbacks:ScaleText(bool)
		PI.Settings.NumAnchor.ScaleText = bool
	end
	
	local Options = PI.Menu.Settings
	
	self.Enabled = Options.UI.CreateHeader(KBM.Language.Options.Enabled[KBM.Lang], PI.Settings, "Enabled", self.Callbacks)
	self.Enabled:CreateCheck(PI.Lang.Options.Fade[KBM.Lang], PI.Settings, "Fade", self.Callbacks)
	self.Enabled:CreateCheck(PI.Lang.Options.Auto[KBM.Lang], PI.Settings, "AutoHide", self.Callbacks)
	self.Visible = self.Enabled:CreateHeader(PI.Lang.Pin.Visible[KBM.Lang], PI.Settings.NumAnchor, "Visible", self.Callbacks)
	self.Visible:CreateCheck(KBM.Language.Options.UnlockWidth[KBM.Lang], PI.Settings.NumAnchor, "ScaleWidth", self.Callbacks)
	self.Visible:CreateCheck(KBM.Language.Options.UnlockHeight[KBM.Lang], PI.Settings.NumAnchor, "ScaleHeight", self.Callbacks)
	self.Visible:CreateCheck(KBM.Language.Options.UnlockText[KBM.Lang], PI.Settings.NumAnchor, "ScaleText", self.Callbacks)
end

function PI.SecureEnter()
	PI.Secure = true
	
	for Index, Frame in ipairs(PI.Frame) do
		Frame.Menu.Settings:SetEnabled(false)
		if KBM.Menu.Current.Page == Frame.Menu.Settings then
			Frame.Menu.Enabled:Disable()
		end
	end
end

function PI.GroupJoin()
	PI.Hide = false
	if not PI.Secure then
		PI:SetPins()
		local UpdateTable = {}
		for UnitID, UnitObj in pairs(LibSUnit.Cache.Avail) do
			if UnitObj.Mark then
				UpdateTable[UnitID] = UnitObj
			end
		end
		PI.MarkChange(handle, UpdateTable)
	end
end

function PI.GroupLeave()
	PI.Hide = PI.Settings.AutoHide
	if not PI.Secure then
		PI:SetPins()
	end
end

function PI.SecureLeave()
	PI.Secure = false
	
	if not LibSUnit.Raid.Grouped then
		PI.Hide = PI.Settings.AutoHide
	else
		PI.Hide = false
	end	
		
	for Index, Frame in ipairs(PI.Frame) do
		Frame.Menu.Settings:SetEnabled(true)
		if KBM.Menu.Current.Page == Frame.Menu.Settings then
			Frame.Menu.Enabled:Enable()
		end
	end
	
	PI:SetPins()
end

function PI:Start()
	self.Secure = Inspect.System.Secure()
	self.Menu.Header = 	KBM.Menu.Page:CreateHeader(self.Name, "KBM_MARKIT", "Main", "Plug")
	self.Menu.Settings = self.Menu.Header:CreateItem(KBM.Language.Options.Settings[KBM.Lang], "KBM_MARKIT_SETTINGS")
	self:InitCastBars()
	self.Menu:BuildOptions()
	self:InitFrameMenus()
	
	if self.Secure then
		self:SecureEnter()
	end
	
	PI.Active = true
		
	Command.Event.Attach(Event.System.Secure.Enter, self.SecureEnter, "Secure Enter")
	Command.Event.Attach(Event.System.Secure.Leave, self.SecureLeave, "Secure Leave")
	Command.Event.Attach(Event.SafesUnitLib.Unit.Mark, self.MarkChange, "Mark Change")
	Command.Event.Attach(Event.SafesUnitLib.Unit.Detail.Percent, self.UpdateUnit, "Unit HP Change")
	Command.Event.Attach(Event.SafesUnitLib.Unit.New.Full, self.UnitAvailable, "Unit Available")
	Command.Event.Attach(Event.SafesUnitLib.Unit.New.Partial, self.UnitAvailable, "Unit Available")
	Command.Event.Attach(Event.SafesUnitLib.Unit.Idle, self.UnitUnavailable, "Unit Unavailable")
	Command.Event.Attach(Event.SafesUnitLib.Unit.Detail.Name, self.UnitName, "Unit Name")
	Command.Event.Attach(Event.SafesUnitLib.Raid.Join, self.GroupJoin, "Join Group")
	Command.Event.Attach(Event.SafesUnitLib.Raid.Leave, self.GroupLeave, "Leave Group")
end